/**
 * Copyright (c) 2008 CDO2 Limited.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of CDO2 Limited nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CDO2 LIMITED ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.scatterspace.remote.group;

import java.util.logging.Level;
import java.util.logging.Logger;

import net.jini.config.Configuration;
import net.jini.config.ConfigurationException;

/**
 * <p>Manages the node's membership of the group, handles incoming requests and
 * co-ordinates with other nodes via the {@link GroupChannel}.</p>
 * 
 * <p>Configured in section <code>net.scatterspace</code>:<ul>
 *  <li><code>groupChannel</code>: a GroupChannel implementation for the
 *      underlying communications</li>
 * </ul></p>
 * 
 * <p>Logs to <code>net.scatterspace.group</code>.</p>
 */
public class GroupNode
{
    private Logger log_ = Logger.getLogger("net.scatterspace.group");
    private GroupChannel channel_;
    
    public GroupNode(Configuration config) throws ConfigurationException
    {
        channel_ = (GroupChannel) config.getEntry(
              "net.scatterspace", "groupChannel", GroupChannel.class);
        
        log_.log(Level.CONFIG,
                 "Group node configured with underlying channel " + channel_);
        
        if (channel_ == null)
            throw new ConfigurationException("groupChannel may not be null");
        
        channel_.setNode(this);
    }
}
