/**
 * Copyright (c) 2008 CDO2 Limited.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of CDO2 Limited nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CDO2 LIMITED ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.scatterspace.storage;

import net.jini.core.entry.Entry;
import net.scatterspace.entry.EntryRep;
import net.scatterspace.op.Operation;

/**
 * Provides storage and retrieval of {@link Entry}s for the space. 
 */
public interface Storage
{
    /**
     * Attempt to locate an {@link Entry} matching the given template
     * <code>tmpl</code> and offer it to the given {@link Operation}
     * <code>op</code>.  The implementation should try and restrict the search
     * time to at least <code>timeout</code> milliseconds.
     * 
     * @param tmpl Entry search template
     * @param op Operation to offer results to
     * @param timeout Max search timeout
     */
    public void search(EntryRep tmpl, Operation op, long timeout);
    
    /**
     * Register an event listener for the arrival of new {@link Entry} items in
     * the {@link Storage} matching the given template <code>tmpl</code>.
     * 
     * @param tmpl Entry template
     * @param listener Listener to notify of {@link Entry} arrivals
     */
    public void registerListener(EntryRep tmpl, StorageListener listener);
}
