/**
 * Copyright (c) 2008 CDO2 Limited.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of CDO2 Limited nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CDO2 LIMITED ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.scatterspace.jmx.jini;

import java.io.IOException;
import java.util.Map;

import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXConnectorProvider;
import javax.management.remote.JMXServiceURL;

import net.jini.core.lookup.ServiceTemplate;
import net.scatterspace.jmx.MBeanConnector;
import net.scatterspace.jmx.SearchURLReader;

/**
 * Implementation of a JMX protocol provider so clients can connect to a
 * remote {@link MBeanConnector} service.  Users should specify system property
 * <code>-Djmx.remote.protocol.provider.pkgs=net.scatterspace.jmx</code> as
 * documented in {@link JMXConnectorFactory}. 
 */
public class ClientProvider implements JMXConnectorProvider
{
    private static final long TIMEOUT__ = 60 * 1000;
    
    /* (non-Javadoc)
     * @see javax.management.remote.JMXConnectorProvider#newJMXConnector(javax.management.remote.JMXServiceURL, java.util.Map)
     */
    public JMXConnector newJMXConnector(JMXServiceURL url, Map<String,?> env)
            throws IOException
    {
        SearchURLReader reader = new SearchURLReader(url);
        
        ServiceTemplate template =
            new ServiceTemplate(reader.getServiceId(),
                                new Class[] { MBeanConnector.class },
                                reader.getAttributes());
        
        long timeout = env.containsKey("net.scatterspace.jmx.timeout")
                ? (Long) env.get("net.scatterspace.jmx.timeout")
                : TIMEOUT__;
        
        return new JiniClientConnector(reader.getDiscoveryGroups(),
                                       template, timeout);
    }
}
