/**
 * Copyright (c) 2008 CDO2 Limited.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of CDO2 Limited nor the names of its contributors
 *       may be used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY CDO2 LIMITED ''AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL <copyright holder> BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package net.scatterspace.remote.group;

import java.util.logging.Level;
import java.util.logging.Logger;

import net.jini.config.Configuration;
import net.scatterspace.remote.group.JGroupsAdapter;
import net.scatterspace.test.ConfigTestCase;



/**
 * Tests {@link JGroupsAdapter}.
 */
public class JGroupsAdapterTest extends ConfigTestCase
{
    private Logger log_ = Logger.getLogger("net.scatterspace.group.jgroups");
    
    private JGroupsAdapter adapter_;
    
    public void setUp() throws Exception
    {
        Configuration config = getConfig("JGroupsAdapterTest.config");
        log_.log(Level.FINE, "Creating adapter to test channel creation");
        
        adapter_ = new JGroupsAdapter(config);
    }
    
    /**
     * Test that a channel can be connected.
     */
    public void testChannelConnect() throws Exception
    {
        int count = connectAndWaitForView(adapter_);
        assertEquals("Member count on A after its connect", 1, count);
    }
    
    /**
     * Test the connection between two separate channel instances.
     */
    public void testSecondNodes() throws Exception
    {
        // Ensure that our first adapter is still working properly before we
        // try and create a second
        int countA = connectAndWaitForView(adapter_);
        assertEquals("Member count on A after its connect", 1, countA);
        
        Configuration config = getConfig("JGroupsAdapterTest.config");
        JGroupsAdapter b = new JGroupsAdapter(config);
        
        int countB = connectAndWaitForView(b);
        assertEquals("Member count on B after its connect", 2, countB);
        assertEquals("Member count on A after B's connect",
                     2, adapter_.getMemberCount());
        
        b.destroy();
    }
    
    /**
     * Connects the adapter and waits for the first view to arrive by monitoring
     * the member count.
     * 
     * @param adapter Adapter to connect
     * @return Number of members in the group
     */
    private int connectAndWaitForView(JGroupsAdapter adapter)
        throws GroupChannelException, InterruptedException
    {
        Object monitor = adapter_.getMemberCountMonitor();
        synchronized (monitor)
        {
            // Monitor the actual member count value for the wait loop
            int originalCount = adapter_.getMemberCount();
            
            adapter.connect();
            
            while (originalCount == adapter_.getMemberCount())
                monitor.wait();
            
            return adapter_.getMemberCount();
        }
    }
    
    public void tearDown() throws Exception
    {
        adapter_.destroy();
    }
}
